/*
 * -- SuperLU routine (version 2.0) --
 * Univ. of California Berkeley, Xerox Palo Alto Research Center,
 * and Lawrence Berkeley National Lab.
 * November 15, 1997
 *
 */

#define BOOST_NUMERIC_BINDINGS_SUPERLU_PRINT

#include <iostream>
#include <complex>
#include <boost/numeric/bindings/superlu/superlu.hpp>
#include <boost/numeric/bindings/traits/ublas_matrix.hpp>
#include <boost/numeric/bindings/traits/ublas_sparse.hpp>

namespace ublas = boost::numeric::ublas;
namespace slu = boost::numeric::bindings::superlu;

typedef ublas::compressed_matrix<double, ublas::column_major, 0,
  ublas::unbounded_array<int>, ublas::unbounded_array<double> > cm_t;
typedef ublas::matrix<double, ublas::column_major> m_t;

int main()
{
/*
 * Purpose
 * =======
 *
 * This is the small 5x5 example used in the Sections 1 and 2 of the
 * User's Guide to illustrate how to call a SuperLU routine, and the
 * matrix data structures used by SuperLU.
 *
 */
    double s, u, p, e, r, l;
    unsigned int m, n;

    /* Initialize matrix A. */
    m = n = 5;
    unsigned int nnz = 12;
    s = 19.0; u = 21.0; p = 16.0; e = 5.0; r = 18.0; l = 12.0;

    /* Create matrix A. */
    cm_t A(m, n, nnz);
    A(0, 0) = s; A(0, 2) = u; A(0, 3) = u;
    A(1, 0) = l; A(1, 1) = u;
    A(2, 1) = l; A(2, 2) = p;
    A(3, 3) = e; A(3, 4) = u;
    A(4, 0) = l; A(4, 1) = l; A(4, 4) = r;

    /* Create right-hand side matrix B. */
    unsigned int nrhs = 1;
    m_t B(m, nrhs);
    for(m_t::size_type i = 0; i < m; ++i) {
	    for(m_t::size_type j = 0; j < nrhs; ++j) {
		    B(i, j) = 1.0;
	    }
    }

#if 0
    int info = slu::gssv (A, B, slu::natural);
#endif
    cm_t L(m, n);
    cm_t U(m, n);
    int info = slu::gstrf (A, perm_c, perm_r, L, U, slu::natural);
    info = slu::gstrf (A, perm_c, perm_r, L, U, slu::natural);
    if(info == 0) {
      info = slu::gstrs(L, U, perm_c, perm_r, B);
    }
}
